@@ -20,7 +20,6 @@ import org.apache.commons.lang3.StringUtils;
 import org.gradle.api.specs.Spec;
 import org.gradle.internal.Cast;
 import org.gradle.internal.Factory;
-import org.gradle.internal.InternalTransformer;
 import org.gradle.internal.IoActions;
 import org.gradle.internal.UncheckedException;
 import org.jspecify.annotations.Nullable;
@@ -426,12 +425,7 @@ public class GUtil {
 
             throw new IllegalArgumentException(
                 String.format("Cannot convert string value '%s' to an enum value of type '%s' (valid case insensitive values: %s)",
-                    literal, enumType.getName(), CollectionUtils.join(", ", CollectionUtils.collect(Arrays.asList(enumType.getEnumConstants()), new InternalTransformer<Object, T>() {
-                        @Override
-                        public String transform(T t) {
-                            return t.name();
-                        }
-                    }))
+                    literal, enumType.getName(), CollectionUtils.join(", ", CollectionUtils.collect(Arrays.asList(enumType.getEnumConstants()), Enum::name))
                 )
             );
         }
