@@ -14,5 +14,5 @@
  * limitations under the License.
  */
 
-@org.gradle.api.NonNullApi
+@org.jspecify.annotations.NullMarked
 package org.gradle.internal.session;
