@@ -16,7 +16,7 @@
 
 package org.gradle.internal.cc.impl
 
-import org.gradle.StartParameter
+import org.gradle.api.internal.StartParameterInternal
 import org.gradle.composite.internal.BuildTreeWorkGraphController
 import org.gradle.internal.build.BuildLifecycleController
 import org.gradle.internal.build.BuildStateRegistry
@@ -39,7 +39,7 @@ class ConfigurationCacheBuildTreeLifecycleControllerFactory internal constructor
     private val cache: BuildTreeConfigurationCache,
     private val taskGraph: BuildTreeWorkGraphController,
     private val stateTransitionControllerFactory: StateTransitionControllerFactory,
-    private val startParameter: StartParameter,
+    private val startParameter: StartParameterInternal,
     private val buildStateRegistry: BuildStateRegistry,
     private val deferredRootBuildGradle: DefaultDeferredRootBuildGradle,
     parameterCarrierFactory: ToolingModelParameterCarrier.Factory,
@@ -79,7 +79,15 @@ class ConfigurationCacheBuildTreeLifecycleControllerFactory internal constructor
         }
 
         val workPreparer = vintageFactory.createWorkPreparer(targetBuild)
-        val workController = ConfigurationCacheAwareBuildTreeWorkController(workPreparer, workExecutor, taskGraph, cache, buildStateRegistry, buildModelParameters)
+        val workController = ConfigurationCacheAwareBuildTreeWorkController(
+            workPreparer,
+            workExecutor,
+            taskGraph,
+            cache,
+            buildStateRegistry,
+            buildModelParameters,
+            startParameter.configurationCacheHeapDumpDir
+        )
 
         val defaultModelCreator = vintageFactory.createModelCreator(targetBuild)
         val modelCreator = ConfigurationCacheAwareBuildTreeModelCreator(defaultModelCreator, cache)
