@@ -15,31 +15,38 @@
  */
 package org.gradle.api.internal.project;
 
-import com.google.common.collect.Maps;
+import com.google.common.collect.Iterables;
 import org.gradle.api.Project;
 import org.gradle.api.artifacts.component.BuildIdentifier;
 import org.gradle.api.artifacts.component.ProjectComponentIdentifier;
 import org.gradle.api.internal.artifacts.DefaultProjectComponentIdentifier;
 import org.gradle.api.internal.initialization.ClassLoaderScope;
-import org.gradle.initialization.DefaultProjectDescriptor;
+import org.gradle.initialization.ProjectDescriptorInternal;
+import org.gradle.initialization.ProjectDescriptorRegistry;
 import org.gradle.internal.Describables;
 import org.gradle.internal.DisplayName;
 import org.gradle.internal.Factories;
 import org.gradle.internal.Factory;
 import org.gradle.internal.build.BuildProjectRegistry;
 import org.gradle.internal.build.BuildState;
+import org.gradle.internal.concurrent.CompositeStoppable;
+import org.gradle.internal.lazy.Lazy;
 import org.gradle.internal.model.CalculatedModelValue;
 import org.gradle.internal.model.ModelContainer;
 import org.gradle.internal.model.StateTransitionControllerFactory;
 import org.gradle.internal.resources.ProjectLeaseRegistry;
 import org.gradle.internal.resources.ResourceLock;
+import org.gradle.internal.service.ServiceRegistry;
 import org.gradle.internal.work.WorkerLeaseService;
 import org.gradle.util.Path;
+import org.jspecify.annotations.Nullable;
 
-import javax.annotation.Nullable;
+import java.io.Closeable;
 import java.io.File;
 import java.util.Collection;
 import java.util.Comparator;
+import java.util.HashMap;
+import java.util.LinkedHashMap;
 import java.util.Map;
 import java.util.Set;
 import java.util.TreeSet;
@@ -48,26 +55,26 @@ import java.util.concurrent.locks.ReentrantLock;
 import java.util.function.Consumer;
 import java.util.function.Function;
 
-public class DefaultProjectStateRegistry implements ProjectStateRegistry {
+public class DefaultProjectStateRegistry implements ProjectStateRegistry, Closeable {
     private final WorkerLeaseService workerLeaseService;
     private final Object lock = new Object();
-    private final Map<Path, ProjectStateImpl> projectsByPath = Maps.newLinkedHashMap();
-    private final Map<ProjectComponentIdentifier, ProjectStateImpl> projectsById = Maps.newHashMap();
-    private final Map<BuildIdentifier, DefaultBuildProjectRegistry> projectsByBuild = Maps.newHashMap();
+    private final Map<Path, ProjectStateImpl> projectsByPath = new LinkedHashMap<>();
+    private final Map<ProjectComponentIdentifier, ProjectStateImpl> projectsById = new HashMap<>();
+    private final Map<BuildIdentifier, DefaultBuildProjectRegistry> projectsByBuild = new HashMap<>();
 
     public DefaultProjectStateRegistry(WorkerLeaseService workerLeaseService) {
         this.workerLeaseService = workerLeaseService;
     }
 
     @Override
-    public void registerProjects(BuildState owner, ProjectRegistry<DefaultProjectDescriptor> projectRegistry) {
-        Set<DefaultProjectDescriptor> allProjects = projectRegistry.getAllProjects();
+    public void registerProjects(BuildState owner, ProjectDescriptorRegistry projectRegistry) {
+        Set<ProjectDescriptorInternal> allProjects = projectRegistry.getAllProjects();
         synchronized (lock) {
             DefaultBuildProjectRegistry buildProjectRegistry = getBuildProjectRegistry(owner);
             if (!buildProjectRegistry.projectsByPath.isEmpty()) {
                 throw new IllegalStateException("Projects for " + owner.getDisplayName() + " have already been registered.");
             }
-            for (DefaultProjectDescriptor descriptor : allProjects) {
+            for (ProjectDescriptorInternal descriptor : allProjects) {
                 addProject(owner, buildProjectRegistry, descriptor);
             }
         }
@@ -83,23 +90,42 @@ public class DefaultProjectStateRegistry implements ProjectStateRegistry {
     }
 
     @Override
-    public ProjectState registerProject(BuildState owner, DefaultProjectDescriptor projectDescriptor) {
+    public ProjectState registerProject(BuildState owner, ProjectDescriptorInternal projectDescriptor) {
         synchronized (lock) {
             DefaultBuildProjectRegistry buildProjectRegistry = getBuildProjectRegistry(owner);
             return addProject(owner, buildProjectRegistry, projectDescriptor);
         }
     }
 
-    private ProjectState addProject(BuildState owner, DefaultBuildProjectRegistry projectRegistry, DefaultProjectDescriptor descriptor) {
+    @Override
+    public void discardProjectsFor(BuildState build) {
+        DefaultBuildProjectRegistry registry = projectsByBuild.get(build.getBuildIdentifier());
+        if (registry != null) {
+            for (ProjectStateImpl project : registry.projectsByPath.values()) {
+                projectsById.remove(project.getComponentIdentifier());
+                projectsByPath.remove(project.getIdentityPath());
+            }
+            CompositeStoppable.stoppable(registry.projectsByPath.values()).stop();
+            registry.projectsByPath.clear();
+        }
+    }
+
+    private ProjectState addProject(BuildState owner, DefaultBuildProjectRegistry projectRegistry, ProjectDescriptorInternal descriptor) {
         Path projectPath = descriptor.path();
-        Path identityPath = owner.calculateIdentityPathForProject(projectPath);
-        String name = descriptor.getName();
-        ProjectComponentIdentifier projectIdentifier = new DefaultProjectComponentIdentifier(owner.getBuildIdentifier(), identityPath, projectPath, name);
-        IProjectFactory projectFactory = owner.getMutableModel().getServices().get(IProjectFactory.class);
-        StateTransitionControllerFactory stateTransitionControllerFactory = owner.getMutableModel().getServices().get(StateTransitionControllerFactory.class);
-        ProjectStateImpl projectState = new ProjectStateImpl(owner, identityPath, projectPath, descriptor.getName(), projectIdentifier, descriptor, projectFactory, stateTransitionControllerFactory);
-        projectsByPath.put(identityPath, projectState);
-        projectsById.put(projectIdentifier, projectState);
+
+        ProjectIdentity identity;
+        if (projectPath.equals(Path.ROOT)) {
+            identity = ProjectIdentity.forRootProject(owner.getIdentityPath(), descriptor.getName());
+        } else {
+            identity = ProjectIdentity.forSubproject(owner.getIdentityPath(), projectPath);
+        }
+
+        ServiceRegistry buildServices = owner.getMutableModel().getServices();
+        IProjectFactory projectFactory = buildServices.get(IProjectFactory.class);
+        StateTransitionControllerFactory stateTransitionControllerFactory = buildServices.get(StateTransitionControllerFactory.class);
+        ProjectStateImpl projectState = new ProjectStateImpl(owner, identity, descriptor, projectFactory, stateTransitionControllerFactory, buildServices);
+        projectsByPath.put(identity.getBuildTreePath(), projectState);
+        projectsById.put(projectState.getComponentIdentifier(), projectState);
         projectRegistry.add(projectPath, projectState);
         return projectState;
     }
@@ -128,6 +154,24 @@ public class DefaultProjectStateRegistry implements ProjectStateRegistry {
         }
     }
 
+    @Override
+    public ProjectState stateFor(Path identityPath) {
+        synchronized (lock) {
+            ProjectStateImpl projectState = projectsByPath.get(identityPath);
+            if (projectState == null) {
+                throw new IllegalArgumentException(identityPath.asString() + " not found.");
+            }
+            return projectState;
+        }
+    }
+
+    @Override
+    public @Nullable ProjectState findProjectState(Path identityPath) {
+        synchronized (lock) {
+            return projectsByPath.get(identityPath);
+        }
+    }
+
     @Override
     public BuildProjectRegistry projectsFor(BuildIdentifier buildIdentifier) throws IllegalArgumentException {
         synchronized (lock) {
@@ -139,15 +183,28 @@ public class DefaultProjectStateRegistry implements ProjectStateRegistry {
         }
     }
 
+    @Nullable
+    @Override
+    public BuildProjectRegistry findProjectsFor(BuildIdentifier buildIdentifier) {
+        synchronized (lock) {
+            return projectsByBuild.get(buildIdentifier);
+        }
+    }
+
     @Override
     public <T> T allowUncontrolledAccessToAnyProject(Factory<T> factory) {
         return workerLeaseService.allowUncontrolledAccessToAnyProject(factory);
     }
 
+    @Override
+    public void close() {
+        CompositeStoppable.stoppable(projectsByPath.values()).stop();
+    }
+
     private static class DefaultBuildProjectRegistry implements BuildProjectRegistry {
         private final BuildState owner;
         private final WorkerLeaseService workerLeaseService;
-        private final Map<Path, ProjectStateImpl> projectsByPath = Maps.newLinkedHashMap();
+        private final Map<Path, ProjectStateImpl> projectsByPath = new LinkedHashMap<>();
 
         public DefaultBuildProjectRegistry(BuildState owner, WorkerLeaseService workerLeaseService) {
             this.owner = owner;
@@ -198,49 +255,45 @@ public class DefaultProjectStateRegistry implements ProjectStateRegistry {
         }
     }
 
-    private class ProjectStateImpl implements ProjectState {
-        private final Path projectPath;
-        private final String projectName;
-        private final ProjectComponentIdentifier identifier;
-        private final DefaultProjectDescriptor descriptor;
+    private class ProjectStateImpl implements ProjectState, Closeable {
+
+        private final ProjectDescriptorInternal descriptor;
         private final IProjectFactory projectFactory;
         private final BuildState owner;
-        private final Path identityPath;
+        private final ProjectIdentity identity;
         private final ResourceLock allProjectsLock;
         private final ResourceLock projectLock;
         private final ResourceLock taskLock;
         private final Set<Thread> canDoAnythingToThisProject = new CopyOnWriteArraySet<>();
         private final ProjectLifecycleController controller;
+        private final Lazy<Integer> depth = Lazy.unsafe().of(() -> getParent() != null ? getParent().getDepth() + 1 : 0);
 
         ProjectStateImpl(
             BuildState owner,
-            Path identityPath,
-            Path projectPath,
-            String projectName,
-            ProjectComponentIdentifier identifier,
-            DefaultProjectDescriptor descriptor,
+            ProjectIdentity identity,
+            ProjectDescriptorInternal descriptor,
             IProjectFactory projectFactory,
-            StateTransitionControllerFactory stateTransitionControllerFactory
+            StateTransitionControllerFactory stateTransitionControllerFactory,
+            ServiceRegistry buildServices
         ) {
             this.owner = owner;
-            this.identityPath = identityPath;
-            this.projectPath = projectPath;
-            this.projectName = projectName;
-            this.identifier = identifier;
             this.descriptor = descriptor;
             this.projectFactory = projectFactory;
+            this.identity = identity;
             this.allProjectsLock = workerLeaseService.getAllProjectsLock(owner.getIdentityPath());
-            this.projectLock = workerLeaseService.getProjectLock(owner.getIdentityPath(), identityPath);
-            this.taskLock = workerLeaseService.getTaskExecutionLock(owner.getIdentityPath(), identityPath);
-            this.controller = new ProjectLifecycleController(getDisplayName(), stateTransitionControllerFactory);
+            this.projectLock = workerLeaseService.getProjectLock(owner.getIdentityPath(), identity.getBuildTreePath());
+            this.taskLock = workerLeaseService.getTaskExecutionLock(owner.getIdentityPath(), identity.getBuildTreePath());
+            this.controller = new ProjectLifecycleController(getDisplayName(), stateTransitionControllerFactory, buildServices);
         }
 
         @Override
         public DisplayName getDisplayName() {
-            if (projectPath.equals(Path.ROOT)) {
-                return Describables.quoted("root project", projectName);
+            Path identityPath = identity.getBuildTreePath();
+            if (identityPath.equals(Path.ROOT)) {
+                return Describables.withTypeAndName("root project", identity.getProjectName());
+            } else {
+                return Describables.withTypeAndName("project", identityPath.asString());
             }
-            return Describables.of(identifier);
         }
 
         @Override
@@ -256,6 +309,7 @@ public class DefaultProjectStateRegistry implements ProjectStateRegistry {
         @Nullable
         @Override
         public ProjectState getParent() {
+            Path identityPath = identity.getBuildTreePath();
             return identityPath.getParent() == null ? null : projectsByPath.get(identityPath.getParent());
         }
 
@@ -265,10 +319,11 @@ public class DefaultProjectStateRegistry implements ProjectStateRegistry {
             if (descriptor.getParent() != null) {
                 // Identity path of parent can be different to identity path parent, if the names are tweaked in the settings file
                 // Ideally they would be exactly the same, always
-                Path parentPath = owner.calculateIdentityPathForProject(descriptor.getParent().path());
-                ProjectStateImpl parentState = projectsByPath.get(parentPath);
+                Path parentProjectPath = descriptor.getParent().path();
+                Path parentIdentityPath = ProjectIdentity.computeProjectIdentityPath(owner.getIdentityPath(), parentProjectPath);
+                ProjectStateImpl parentState = projectsByPath.get(parentIdentityPath);
                 if (parentState == null) {
-                    throw new IllegalStateException("Parent project " + parentPath + " is not registered for project " + identityPath);
+                    throw new IllegalStateException("Parent project " + parentIdentityPath + " is not registered for " + identity);
                 }
                 return parentState;
             } else {
@@ -279,25 +334,46 @@ public class DefaultProjectStateRegistry implements ProjectStateRegistry {
         @Override
         public Set<ProjectState> getChildProjects() {
             Set<ProjectState> children = new TreeSet<>(Comparator.comparing(ProjectState::getIdentityPath));
-            for (DefaultProjectDescriptor child : descriptor.children()) {
-                children.add(projectsByPath.get(owner.calculateIdentityPathForProject(child.path())));
+            for (ProjectDescriptorInternal child : descriptor.children()) {
+                children.add(getStateForChild(child));
             }
             return children;
         }
 
+        @Override
+        public Iterable<ProjectState> getUnorderedChildProjects() {
+            return Iterables.transform(descriptor.children(), this::getStateForChild);
+        }
+
+        @Override
+        public boolean hasChildren() {
+            return !descriptor.children().isEmpty();
+        }
+
+        private ProjectStateImpl getStateForChild(ProjectDescriptorInternal child) {
+            Path childProjectPath = child.path();
+            Path childIdentityPath = ProjectIdentity.computeProjectIdentityPath(owner.getIdentityPath(), childProjectPath);
+            return projectsByPath.get(childIdentityPath);
+        }
+
         @Override
         public String getName() {
-            return projectName;
+            return identity.getProjectName();
         }
 
         @Override
         public Path getIdentityPath() {
-            return identityPath;
+            return identity.getBuildTreePath();
+        }
+
+        @Override
+        public ProjectIdentity getIdentity() {
+            return identity;
         }
 
         @Override
         public Path getProjectPath() {
-            return projectPath;
+            return identity.getProjectPath();
         }
 
         @Override
@@ -305,6 +381,16 @@ public class DefaultProjectStateRegistry implements ProjectStateRegistry {
             return descriptor.getProjectDir();
         }
 
+        @Override
+        public int getDepth() {
+            return depth.get();
+        }
+
+        @Override
+        public boolean isCreated() {
+            return controller.isCreated();
+        }
+
         @Override
         public void createMutableModel(ClassLoaderScope selfClassLoaderScope, ClassLoaderScope baseClassLoaderScope) {
             controller.createMutableModel(descriptor, owner, this, selfClassLoaderScope, baseClassLoaderScope, projectFactory);
@@ -325,6 +411,15 @@ public class DefaultProjectStateRegistry implements ProjectStateRegistry {
             controller.ensureSelfConfigured();
         }
 
+        @Override
+        public void ensureSelfConfigured() {
+            ProjectState parent = getBuildParent();
+            if (parent != null) {
+                ((ProjectStateImpl) parent).controller.assertConfigured();
+            }
+            controller.ensureSelfConfigured();
+        }
+
         @Override
         public void ensureTasksDiscovered() {
             controller.ensureTasksDiscovered();
@@ -332,7 +427,7 @@ public class DefaultProjectStateRegistry implements ProjectStateRegistry {
 
         @Override
         public ProjectComponentIdentifier getComponentIdentifier() {
-            return identifier;
+            return new DefaultProjectComponentIdentifier(identity);
         }
 
         @Override
@@ -402,6 +497,11 @@ public class DefaultProjectStateRegistry implements ProjectStateRegistry {
         public <T> CalculatedModelValue<T> newCalculatedValue(@Nullable T initialValue) {
             return new CalculatedModelValueImpl<>(this, workerLeaseService, initialValue);
         }
+
+        @Override
+        public void close() {
+            controller.close();
+        }
     }
 
     private static class CalculatedModelValueImpl<T> implements CalculatedModelValue<T> {
