@@ -29,7 +29,8 @@ sealed interface SchemaItemMetadata : Serializable
 
 @ToolingModelContract(
     subTypes = [
-        ContainerElementFactory::class
+        ContainerElementFactory::class,
+        SoftwareFeatureOrigin::class
     ]
 )
 sealed interface SchemaMemberOrigin : SchemaItemMetadata
@@ -37,3 +38,13 @@ sealed interface SchemaMemberOrigin : SchemaItemMetadata
 interface ContainerElementFactory : SchemaMemberOrigin {
     val elementType: DataTypeRef
 }
+
+interface SoftwareFeatureOrigin : SchemaMemberOrigin {
+    val featureName: String
+    val featurePluginClassName: String
+    val ecosystemPluginClassName: String
+    val ecosystemPluginId: String?
+    val targetDefinitionClassName: String?
+    val targetBuildModelClassName: String?
+    //TODO: feature owner plugin ID?
+}
