@@ -31,12 +31,14 @@ import org.gradle.api.attributes.AttributesSchema;
 import org.gradle.api.file.Directory;
 import org.gradle.api.file.ProjectLayout;
 import org.gradle.api.internal.CollectionCallbackActionDecorator;
+import org.gradle.api.internal.ConfigurationServicesBundle;
 import org.gradle.api.internal.DocumentationRegistry;
 import org.gradle.api.internal.DomainObjectContext;
 import org.gradle.api.internal.GradleInternal;
 import org.gradle.api.internal.artifacts.configurations.ConfigurationContainerInternal;
 import org.gradle.api.internal.artifacts.configurations.DefaultConfigurationContainer;
 import org.gradle.api.internal.artifacts.configurations.DefaultConfigurationFactory;
+import org.gradle.api.internal.artifacts.configurations.DefaultConfigurationServicesBundle;
 import org.gradle.api.internal.artifacts.configurations.DependencyMetaDataProvider;
 import org.gradle.api.internal.artifacts.configurations.ResolutionStrategyFactory;
 import org.gradle.api.internal.artifacts.dsl.ComponentMetadataHandlerInternal;
@@ -93,9 +95,11 @@ import org.gradle.api.internal.artifacts.transform.TransformRegistrationFactory;
 import org.gradle.api.internal.artifacts.transform.TransformedVariantFactory;
 import org.gradle.api.internal.artifacts.type.ArtifactTypeRegistry;
 import org.gradle.api.internal.attributes.AttributeDescriberRegistry;
+import org.gradle.api.internal.attributes.AttributeDesugaring;
 import org.gradle.api.internal.attributes.AttributesFactory;
 import org.gradle.api.internal.attributes.AttributesSchemaInternal;
 import org.gradle.api.internal.attributes.DefaultAttributesSchema;
+import org.gradle.api.internal.collections.DomainObjectCollectionFactory;
 import org.gradle.api.internal.file.FileCollectionFactory;
 import org.gradle.api.internal.file.FileLookup;
 import org.gradle.api.internal.file.FilePropertyFactory;
@@ -103,6 +107,7 @@ import org.gradle.api.internal.file.FileResolver;
 import org.gradle.api.internal.model.NamedObjectInstantiator;
 import org.gradle.api.internal.project.ProjectStateRegistry;
 import org.gradle.api.internal.provider.PropertyFactory;
+import org.gradle.api.internal.tasks.TaskDependencyFactory;
 import org.gradle.api.model.ObjectFactory;
 import org.gradle.api.problems.internal.InternalProblems;
 import org.gradle.api.provider.ProviderFactory;
@@ -177,6 +182,7 @@ public class DefaultDependencyManagementServices implements DependencyManagement
         FileCollectionFactory fileCollectionFactory,
         DomainObjectContext owner
     ) {
+        @SuppressWarnings("resource")
         ServiceRegistry services = ServiceRegistryBuilder.builder()
             .parent(parent)
             .provider(registration -> {
@@ -577,6 +583,35 @@ public class DefaultDependencyManagementServices implements DependencyManagement
             };
         }
 
+        @Provides
+        ConfigurationServicesBundle createConfigurationServicesBundle(BuildOperationRunner buildOperationRunner,
+                                                                      ProjectStateRegistry projectStateRegistry,
+                                                                      FileCollectionFactory fileCollectionFactory,
+                                                                      ObjectFactory objectFactory,
+                                                                      AttributesFactory attributesFactory,
+                                                                      CollectionCallbackActionDecorator collectionCallbackActionDecorator,
+                                                                      DomainObjectCollectionFactory domainObjectCollectionFactory,
+                                                                      CalculatedValueContainerFactory calculatedValueContainerFactory,
+                                                                      TaskDependencyFactory taskDependencyFactory,
+                                                                      InternalProblems problems,
+                                                                      AttributeDesugaring attributeDesugaring,
+                                                                      ResolveExceptionMapper exceptionMapper) {
+            return new DefaultConfigurationServicesBundle(
+                buildOperationRunner,
+                projectStateRegistry,
+                calculatedValueContainerFactory,
+                objectFactory,
+                fileCollectionFactory,
+                taskDependencyFactory,
+                attributesFactory,
+                domainObjectCollectionFactory,
+                collectionCallbackActionDecorator,
+                problems,
+                attributeDesugaring,
+                exceptionMapper
+            );
+        }
+
         private static List<ResolutionAwareRepository> collectRepositories(RepositoryHandler repositoryHandler) {
             return repositoryHandler.stream()
                 .map(ResolutionAwareRepository.class::cast)
