@@ -27,9 +27,12 @@ import org.gradle.api.internal.attributes.immutable.ImmutableAttributesSchema;
 import org.gradle.api.internal.attributes.matching.AttributeMatcher;
 import org.gradle.internal.collections.ImmutableFilteredList;
 import org.gradle.internal.lazy.Lazy;
+import org.gradle.internal.service.scopes.Scope;
+import org.gradle.internal.service.scopes.ServiceScope;
+import org.jspecify.annotations.Nullable;
 
-import javax.annotation.Nullable;
 import java.util.ArrayList;
+import java.util.Collections;
 import java.util.List;
 import java.util.concurrent.ConcurrentHashMap;
 import java.util.function.BiFunction;
@@ -42,6 +45,7 @@ import java.util.function.BiFunction;
  * Caches the results, as often the same request is made for many components in a
  * dependency graph.
  */
+@ServiceScope(Scope.Project.class)
 public class ConsumerProvidedVariantFinder {
     private final VariantTransformRegistry variantTransforms;
     private final AttributesFactory attributesFactory;
@@ -174,11 +178,13 @@ public class ConsumerProvidedVariantFinder {
                 // Construct new states for processing at the next depth in case we can't find any solutions at this depth.
                 for (int i = 0; i < candidates.size(); i++) {
                     TransformRegistration candidate = candidates.get(i);
-                    nextDepth.add(new ChainState(
-                        new ChainNode(state.chain, candidate),
-                        attributesFactory.concat(state.requested, candidate.getFrom()),
-                        state.transforms.withoutIndexFrom(i, candidates)
-                    ));
+                    if (!Collections.disjoint(state.requested.keySet(), candidate.getTo().keySet())) {
+                        nextDepth.add(new ChainState(
+                            new ChainNode(state.chain, candidate),
+                            attributesFactory.concat(state.requested, candidate.getFrom()),
+                            state.transforms.withoutIndexFrom(i, candidates)
+                        ));
+                    }
                 }
             }
 
@@ -233,7 +239,7 @@ public class ConsumerProvidedVariantFinder {
         ) {
             List<ImmutableAttributes> variantAttributes = new ArrayList<>(sources.size());
             for (ResolvedVariant variant : sources) {
-                variantAttributes.add(variant.getAttributes().asImmutable());
+                variantAttributes.add(variant.getAttributes());
             }
             List<CachedVariant> cached = cache.computeIfAbsent(new CacheKey(variantAttributes, requested), key -> action.apply(key.variantAttributes, key.requested));
             List<TransformedVariant> output = new ArrayList<>(cached.size());
