@@ -16,13 +16,19 @@
 
 package org.gradle.internal.scan.config;
 
+import org.gradle.internal.service.scopes.Scope;
+import org.gradle.internal.service.scopes.ServiceScope;
+
 /**
- * A service that provides the build scan configuration.
- *
+ * A check-in service used by the Gradle Enterprise plugin versions until 3.4, none of which are supported anymore.
+ * <p>
+ * We keep this service, because for the plugin versions 3.0+ we can gracefully avoid plugin application and report an unsupported message.
+ * <p>
  * Obtained via the root project's gradle object's service registry.
  *
  * @since 4.0
  */
+@ServiceScope(Scope.Build.class)
 public interface BuildScanConfigProvider {
 
     /**
