@@ -16,8 +16,12 @@
 
 package org.gradle.internal.logging.progress;
 
+import org.gradle.internal.service.scopes.Scope;
+import org.gradle.internal.service.scopes.ServiceScope;
+
 import java.io.IOException;
 
+@ServiceScope(Scope.Project.class)
 public interface ProgressLoggingInputStreamListener {
 
     void onProcessedBytes(int numberOfBytes) throws IOException;
