@@ -18,7 +18,6 @@ package org.gradle.internal.snapshot.impl;
 
 import com.google.common.collect.ImmutableList;
 import org.gradle.internal.isolation.Isolatable;
-import org.jspecify.annotations.Nullable;
 
 import java.util.Properties;
 
@@ -27,7 +26,6 @@ public class IsolatedProperties extends AbstractIsolatedMap<Properties> {
         super(entries);
     }
 
-    @Nullable
     @Override
     public Properties create() {
         return new Properties();
