@@ -15,7 +15,8 @@
  */
 package org.gradle.cache;
 
-import javax.annotation.Nullable;
+import org.jspecify.annotations.Nullable;
+
 import java.util.function.Function;
 import java.util.function.Supplier;
 
