@@ -17,7 +17,7 @@
 /**
  * Exceptions installing JVM toolchains.
  */
-@NonNullApi
+@NullMarked
 package org.gradle.jvm.toolchain.internal.install.exceptions;
 
-import org.gradle.api.NonNullApi;
+import org.jspecify.annotations.NullMarked;
